#pragma once

#include "..\..\DuiLib\UIlib.h"
#include "..\DuiLib\Core\UIDefine.h"
#include "EmbeddedDUIInMFCWindow.h"


class IDUIEventCallBack;
template<class T>
class CDUIControl : public EmbeddedDUIInMFCWindow
{
	typename typedef T CControl;
public:
	CDUIControl(CWnd* pWnd, IDUIEventCallBack* pCallBack)
		:EmbeddedDUIInMFCWindow(pWnd)
	{
		m_pCallBack = pCallBack;
	}
	virtual ~CDUIControl(void)
	{
		
	}

	HWND GetSafeHwnd()
	{
		return this == NULL ? NULL : GetHWND();
	}
	CControl* GetControl()
	{
		return m_pControl;
	}

	LPCTSTR GetWindowClassName() const
	{
		return _T("DUIControl");
	}

	virtual void Notify(TNotifyUI& msg)
	{
		ASSERT(m_pCallBack);
		if(m_pCallBack != NULL)
		{
			m_pCallBack->DUINotify(msg);
		}
	}

	virtual LRESULT OnNcHitTest(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		return HTCLIENT;
	}

	virtual CDuiString GetSkinFile()
	{
		return _T("DUIControl.xml");
	}

	virtual CDuiString GetSkinFolder()
	{
		return _T("skin\\nTalkRes");
	}

	void LoadSkin()
	{
		m_PaintManager.Init(m_hWnd);
		CDialogBuilder dlgBuilder;
		CDialogBuilderCallback<CDUIControl> cb(this);
		CControlUI* pRoot = dlgBuilder.Create(_T("DUIControl.xml"), (UINT)0,  &cb, &m_PaintManager);
		ASSERT(pRoot);
		m_PaintManager.AttachDialog(pRoot);
		m_PaintManager.AddNotifier(this);

		m_pControl = CreateInnerControl();
	}

	virtual void OnFinalMessage(HWND hWnd)
	{
		WindowImplBase::OnFinalMessage(hWnd);
	}

	virtual CControlUI* CreateControl(LPCTSTR pstrClass)
	{
		return NULL;
	}

private:

	CControl* CreateInnerControl()
	{
		CControl* pControl = new CControl();
		CHorizontalLayoutUI* pLayout = static_cast<CHorizontalLayoutUI*>(m_PaintManager.FindControl(_T("container")));
		ASSERT(pLayout);

		if(pLayout != NULL)
		{
			pLayout->Add(pControl);
		}

		return pControl;
	}

	CControl* m_pControl;
	IDUIEventCallBack* m_pCallBack;
};


class IDUIEventCallBack
{
public:
	virtual void DUINotify(TNotifyUI& msg) = 0;
	virtual ~IDUIEventCallBack(){}
};

